#include "OSTPPKGameModule.h"
#include "IProxy.h"
#include "IPacket.h"
#include <shlwapi.h>
#include <stdio.h>
#pragma comment(lib, "Shlwapi.lib")
#include "command.h"
#include "Item.h"
#include "ItemReader.h"
#include "BitField.h"

OSTPPKGameModule::OSTPPKGameModule(IProxy* proxy) :
	_state(ChickenState_InTown),
	_proxy(proxy)
{
	_hero.id = 0;
	_hero.name[0] = '\0';
	_hero.level = 0;
	_chatName = "c2OSTPPK";
}
void __stdcall OSTPPKGameModule::Destroy()
{
	delete this;
}
OSTPPKGameModule::~OSTPPKGameModule()
{
}

void OSTPPKGameModule::OnRelayDataToServer(IPacket* packet, const IModule* owner)
{
	const unsigned char* bytes = static_cast<const unsigned char*>(packet->GetData());
	int packetId = bytes[0];

		switch (packetId)
	{
	case 0x68:
		{
			// store off the player's name for future reference
			const char* playerName = reinterpret_cast<const char*>(bytes + 21);
			strcpy_s(_hero.name, sizeof(_hero.name), playerName);
		}
		break;

		case 0x3f:
		{
			int soundId = *reinterpret_cast<const short*>(bytes + 1);
			switch(soundId)	
			{
			case 0x1B:
				{
					packet->SetFlag(IPacket::PacketFlag_Virtual);
					_state = ChickenState_OpeningTp;
					_hostile = true;
					CastTp();
				}
				break;

				case 0x1a:
				{
					packet->SetFlag(IPacket::PacketFlag_Virtual);
					int size = _players.size();
					if (currentIter < (size - 1))
					{
						currentIter++;
					}
					else
					{
						currentIter = 0;
					}
					std::list<Player>::iterator i = _players.begin();
					std::advance(i, currentIter);
					_currentTarget = i->id;
					Chat(_chatName, false, "Currently targeting c1%s", i->name);

				}
				break;

				case 0x1d:
					{
						packet->SetFlag(IPacket::PacketFlag_Virtual);
						UseLeftSkillOnObject(0x00, _currentTarget);
						//Chat("Left", false, "AA: %i", _currentTarget);
					}
					break;
				case 0x1e:
					{
						packet->SetFlag(IPacket::PacketFlag_Virtual);
						UseRightSkillOnObject(0x00, _currentTarget);
						//Chat("Right", false, "AA");
					}
					break;
			}
		}
		break;
	}
	
}

void OSTPPKGameModule::OnRelayDataToClient(IPacket* packet, const IModule* owner)
{
	const unsigned char* bytes = static_cast<const unsigned char*>(packet->GetData());
	int packetId = bytes[0];




	switch (packetId)
	{

	case 0x96:
		{
			unsigned char* buffer = new unsigned char[packet->GetSize()];
			memcpy(buffer, packet->GetData(), packet->GetSize());
			void* data = reinterpret_cast<void*>(buffer);
			BitField field(data, sizeof(data));
			WORD xPos = (WORD)(*(DWORD*)(&bytes[2])>>7);
			WORD yPos = (WORD)(*(DWORD*)(&bytes[4])>>7);
			_hero.xPos = xPos;
			_hero.yPos= yPos;

			delete[] buffer;
		}
		break;

	case 0x9c:
		{
			unsigned char* buffer = new unsigned char[packet->GetSize()];
			memcpy(buffer, packet->GetData(), packet->GetSize());
			item thisItem = ItemReader::ItemReader().Read(buffer);
			delete[] buffer;
			if (_stricmp(thisItem.BaseItem.c_str(), "tbk") == 0)
			{
				//Chat(_chatName, false, "Item destination: %i, Item container: %i", thisItem.Destination, thisItem.Container);
				if (thisItem.Action == PutInContainer && thisItem.Destination == Unspecified && thisItem.Container == Inventory)
				{
					TownPortalBook = thisItem;
				}
			}
		}
		break;


	case 0x5a:
		{
			int action = bytes[1];

			// if a player is leaving the game then find out their name and
			// remove them from the module's list of in-game players
			if (action == 0x03)
			{
				const char* playerName = reinterpret_cast<const char*>(bytes + 8);
				RemovePlayer(playerName);
			}

			if (action == 0x06)
			{
				const char* playerName = reinterpret_cast<const char*>(bytes + 8);
				if (_stricmp(playerName, _hero.name) == 0)
				{
					_state = ChickenState_InTown;
				}
			}
		}
		break;

	case 0x5b:
		{
			int playerId = *reinterpret_cast<const int*>(bytes + 3);
			int playerLevel = *reinterpret_cast<const short*>(bytes + 24);
			const char* playerName = reinterpret_cast<const char*>(bytes + 8);

			// a player has entered the game; if they are the hero then store
			// off additional information about them (like their level and id)
			if (strcmp(playerName, _hero.name) == 0)
			{
				_hero.id = playerId;
				_hero.level = playerLevel;
			}
			AddPlayer(playerName, playerLevel, playerId);

		}
		break;

	case 0x82:
		{
			int portalId = *reinterpret_cast<const int*>(bytes + 21);
			int playerId = *reinterpret_cast<const int*>(bytes + 1);

			// the server is notifying us about a town portal that has just come
			// into view/existance; if this town portal belongs to the hero and 
			// the module is trying to open a tp, then use this tp
			if (playerId == _hero.id && _state == ChickenState_OpeningTp)
			{
				UsePortal(portalId);
				_state = ChickenState_UsingTp;
			}
		}
		break;

	case 0x15:
		{
			int unitType = bytes[1];
			if (unitType == 0)
			{
				int playerId = *reinterpret_cast<const int*>(bytes + 2);

				int xPos1 = bytes[6];
				int xPos2 = bytes[7];
				int xPos = (xPos2 << 8) | xPos1;
				int yPos1 = bytes[8];
				int yPos2 = bytes[9];
				int yPos = (yPos2 << 8) | yPos1;
				_hero.xPos = xPos;
				_hero.yPos = yPos;
			//	Chat("Chicken", false, "yPos: %u", xPos);
			//	Chat("Chicken", false, "yPos: %u", yPos);

				// the given player has changed position; if that player is the hero
				// and the module is currently trying to use the tp, then this means
				// that the hero has entered town and should hostile everyone
				if (playerId == _hero.id && _state == ChickenState_UsingTp)
				{			
					if (_hero.level >= 9 && _hostile)
					{
						HostileParty();
					}
					_state = ChickenState_InTown;
					_hostile = false;
				}
			}
		}
		break;

	}
}

void OSTPPKGameModule::Update()
{

}

void OSTPPKGameModule::CastTp()
{
	InvInteract(TownPortalBook.Id);
	//Chat(_chatName, false, "Using tp book: %u", TownPortalBook.Id);
}

void OSTPPKGameModule::UsePortal(int portalId)
{
	// use the town portal object
	UseObject(2, portalId);
}

void OSTPPKGameModule::UseObject(int objectType, int objectId)
{
	unsigned char buffer[9];
	int offset = 0;

	buffer[offset++] = 0x13;
	*reinterpret_cast<int*>(buffer + offset) = objectType;
	offset += sizeof(int);
	*reinterpret_cast<int*>(buffer + offset) = objectId;
	offset += sizeof(int);

	IPacket* packet = _proxy->CreatePacket(buffer, offset);
	_proxy->RelayDataToServer(packet, this);
	delete packet;
}

void OSTPPKGameModule::InvInteract(int objectId)
{
	int offset = 0;
	unsigned char buffer[13];
	buffer[offset++] = 0x20;
	*reinterpret_cast<int*>(buffer+offset) = objectId;
	offset+= sizeof(int);
	*reinterpret_cast<int*>(buffer+offset) = _hero.xPos;
	offset+= sizeof(int);
	*reinterpret_cast<int*>(buffer+offset) = _hero.yPos;
	offset+= sizeof(int);

	IPacket* packet = _proxy->CreatePacket(buffer, offset);
	_proxy->RelayDataToServer(packet, this);
	delete packet;
}


void OSTPPKGameModule::Chat(const char* name, bool whisper, const char* format, ...)
{
	char text[4096];

	va_list arguments;
	va_start(arguments, format);
	vsprintf_s(text, sizeof(text), format, arguments);
	va_end(arguments);	

	int length = static_cast<int>(strlen(text) + strlen(name)) + 12;
	unsigned char* buffer = new unsigned char[length];
	int offset = 0;

	buffer[offset++] = 0x26;
	buffer[offset++] = whisper ? 0x02 : 0x01;
	buffer[offset++] = 0x00;
	buffer[offset++] = 0x02;
	buffer[offset++] = 0x00;
	buffer[offset++] = 0x00;
	buffer[offset++] = 0x00;
	buffer[offset++] = 0x00;
	buffer[offset++] = 0x00;
	buffer[offset++] = whisper ? 0x01 : 0x05;

	for (int i = 0; name[i] != '\0'; i++)
	{
		buffer[offset++] = name[i];
	}
	buffer[offset++] = 0x00;

	for (int i = 0; text[i] != '\0'; i++)
	{
		buffer[offset++] = text[i];
	}
	buffer[offset++] = 0x00;

	IPacket* packet = _proxy->CreatePacket(buffer, length);
	packet->SetFlag(IPacket::PacketFlag_Hidden);
	_proxy->RelayDataToClient(packet, this);
	delete packet;

	delete[] buffer;
}

void OSTPPKGameModule::AddPlayer(const char* playerName, int playerLevel, int playerId)
{
	Player player;

	strcpy_s(player.name, sizeof(player.name), playerName);
	player.id = playerId;
	player.level = playerLevel;
	player.isHostile = 1;

	_players.push_back(player);
}

void OSTPPKGameModule::RemovePlayer(const char* playerName)
{
	// find the player entry that has the same name as the one provided; remove
	// that entry from the list and break out immediately because there won't
	// be any other players with the same name
	for (std::list<Player>::iterator i = _players.begin(); i != _players.end(); i++)
	{
		if (strcmp(playerName, i->name) == 0)
		{
			_players.erase(i);
			break;
		}
	}
}

void OSTPPKGameModule::HostilePlayer(int playerId)
{
	unsigned char buffer[7];
	int offset = 0;

	buffer[offset++] = 0x5d;
	buffer[offset++] = 0x04;
	buffer[offset++] = 0x01;
	*reinterpret_cast<int*>(buffer + offset) = playerId;
	offset += sizeof(int);

	IPacket* packet = _proxy->CreatePacket(buffer, offset);
	_proxy->RelayDataToServer(packet, this);
	delete packet;
}

void OSTPPKGameModule::HostileParty()
{
	// iterate through the entire party and hostile everyone who isn't
	// the player and is of level greater or equal to nine (pvp restriction)
	Chat(_chatName, false, "Hostiling party");
	for (std::list<Player>::iterator i = _players.begin(); i != _players.end(); i++)
	{
		if (i->id != _hero.id && i->level >= 9)
		{
			HostilePlayer(i->id);
			//Chat(_chatName, false, "Hostiling %s", i->name);
		}
	}
}

void OSTPPKGameModule::UseRightSkillOnObject(int objectType, int objectId)
{
	unsigned char buffer[5];
	int offset = 0;

	buffer[offset++] = 0x0d;
	*reinterpret_cast<int*>(buffer + offset) = objectType;
	offset += sizeof(int);
	*reinterpret_cast<int*>(buffer + offset) = objectId;
	offset += sizeof(int);

	IPacket* packet = _proxy->CreatePacket(buffer, offset);
	_proxy->RelayDataToServer(packet, this);
	delete packet;
}

void OSTPPKGameModule::UseLeftSkillOnObject(int objectType, int objectId)
{
	unsigned char buffer[5];
	int offset = 0;

	buffer[offset++] = 0x06;
	*reinterpret_cast<int*>(buffer + offset) = objectType;
	offset += sizeof(int);
	*reinterpret_cast<int*>(buffer + offset) = objectId;
	offset += sizeof(int);

	IPacket* packet = _proxy->CreatePacket(buffer, offset);
	_proxy->RelayDataToServer(packet, this);
	delete packet;
}

